@@ -1,5 +1,5 @@
 tobit2Bfit <- function(YS, XS, YO, XO, start,
-                      print.level=0,
+                      weights = NULL, print.level=0,
                       maxMethod="BHHH",
                       ...) {
    ## Fit 2-dimensional sample selection models where the outcome
@@ -41,22 +41,22 @@ tobit2Bfit <- function(YS, XS, YO, XO, start,
       XO10.b <- drop(XO[i10,] %*% betaO)
       XO11.b <- drop(XO[i11,] %*% betaO)
       lik <- loglik <- numeric(nObs)
-                           # lik is needed for the gradient
+         # lik is the *unweighted* likelihood function needed for the gradient
       ## YS == 0, YO == 0
-      loglik[i00] <- pnorm(-XS00.b, log.p=TRUE)
-      lik[i00] <- exp(loglik[i00])
+      loglik[i00] <- w00 * pnorm(-XS00.b, log.p=TRUE)
+      lik[i00] <- exp(loglik[i00]/w00)
       ## YS == 1, YO == 0
       lowermat <- -cbind(XS10.b, XO10.b)
       f2 <- apply(lowermat, 1,
                   function(x) pmvnorm(lower=x, corr=Sigma))
       lik[i10] <- pnorm(-XS10.b, lower.tail=FALSE) - f2
-      loglik[i10] <- log(lik[i10])
+      loglik[i10] <- w10 * log(lik[i10])
       ## YS == 1, YO == 1
       lowermat <- -cbind(XS11.b, XO11.b)
       f2 <- apply(lowermat, 1,
                   function(x) pmvnorm(lower=x, corr=Sigma))
       lik[i11] <- f2
-      loglik[i11] <- log(f2)
+      loglik[i11] <- w11 * log(f2)
       ## --- gradient ---
       ## The d loglik/d rho was taken from Henry Nyberg "A Bivariate Autoregressive Probit Model:
       ##   Predicting U.S. Business Cycle and Growth Rate Cycle Recessions",
@@ -64,29 +64,29 @@ tobit2Bfit <- function(YS, XS, YO, XO, start,
       grad <- matrix(0, nObs, nParam)
       r <- sqrt(1 - rho^2)
       ## YS == 0, YO == 0
-      grad[i00,ibetaS] <- XS[i00,] * (-dnorm(-XS00.b)/lik[i00])
+      grad[i00,ibetaS] <- w00 * XS[i00,] * (-dnorm(-XS00.b)/lik[i00])
       ## YS == 1, YO == 0
       A <- dnorm(XS10.b)
       B <- A*pnorm((XO10.b - rho*XS10.b)/r, lower.tail=FALSE)
-      grad[i10,ibetaS] <- XS[i10,]*B/lik[i10]
+      grad[i10,ibetaS] <- w10 * XS[i10,]*B/lik[i10]
       A <- dnorm(XO10.b)
       B <- A*pnorm((XS10.b - rho*XO10.b)/r)
-      grad[i10,ibetaO] <- -XO[i10,]*B/lik[i10]
+      grad[i10,ibetaO] <- -w10 * XO[i10,]*B/lik[i10]
       locmat <- -cbind(XS10.b, XO10.b)
       pdf <- apply(locmat, 1,
                    function(x) dmvnorm(x, c(0,0), Sigma))
-      grad[i10,iRho] <- -pdf/lik[i10]
+      grad[i10,iRho] <- -w10 * pdf/lik[i10]
       ## YS == 1, YO == 1
       A <- dnorm(XS11.b)
       B <- A*pnorm((XO11.b - rho*XS11.b)/r)
-      grad[i11,ibetaS] <- XS[i11,]*B/lik[i11]
+      grad[i11,ibetaS] <- w11 * XS[i11,]*B/lik[i11]
       A <- dnorm(XO11.b)
       B <- A*pnorm((XS11.b - rho*XO11.b)/r)
-      grad[i11,ibetaO] <- XO[i11,]*B/lik[i11]
+      grad[i11,ibetaO] <- w11 * XO[i11,]*B/lik[i11]
       locmat <- -cbind(XS11.b, XO11.b)
       pdf <- apply(locmat, 1,
                    function(x) dmvnorm(x, c(0,0), Sigma))
-      grad[i11,iRho] <- pdf/lik[i11]
+      grad[i11,iRho] <- w11 * pdf/lik[i11]
       ## loglik <- sum(loglik)
       ## grad <- colSums(grad)
       attr(loglik, "gradient") <- grad
@@ -114,6 +114,20 @@ tobit2Bfit <- function(YS, XS, YO, XO, start,
    ibetaO <- seq(tail(ibetaS, 1)+1, length=NXO)
    iRho <- tail(ibetaO, 1) + 1
    nParam <- iRho
+   
+   # weights
+   if( !is.null( weights ) ) {
+      w  <- weights
+      w00 <- weights[ i00 ]
+      w10 <- weights[ i10 ]
+      w11 <- weights[ i11 ]
+   } else {
+      w  <- rep( 1, nObs )
+      w00 <- rep( 1, sum( i00 ) )
+      w10 <- rep( 1, sum( i10 ) )
+      w11 <- rep( 1, sum( i11 ) )
+   }
+   
    ## output, if asked for it
    if( print.level > 0) {
       cat("YO observed:", NO, "times; not observed:", nObs - NO,
@@ -122,11 +136,13 @@ tobit2Bfit <- function(YS, XS, YO, XO, start,
       cat( "Initial values:\n")
       print(start)
    }
+   
+   # browser()
+   # compareDerivatives(loglik, gradlik, t0=start )
+   # range(numericGradient(loglik, t0=start)-gradlik(start))
+   
    ## estimate
    library(mvtnorm)
-   ## compareDerivatives(loglik, gradlik,
-   ##                    t0=start)
-   ## stop()
    result <- maxLik(loglik, 
                     start=start,
                     method=maxMethod,
